package com.persagy.apm.alarmservice.alarmrecord.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.alarmservice.alarmcondition.model.vo.AlarmConditionStrItem;
import com.persagy.apm.alarmservice.alarmrecord.enums.EnumAlarmLevel;
import com.persagy.apm.alarmservice.alarmrecord.enums.EnumAlarmState;
import com.persagy.apm.alarmservice.alarmrecord.enums.EnumAlarmTreatMode;
import com.persagy.apm.alarmservice.alarmrecord.enums.EnumAlarmTreatState;
import com.persagy.apm.alarmservice.alarmrecord.model.vo.AlarmRecordItemVO;
import com.persagy.apm.alarmservice.alarmrecord.model.vo.AlarmRecordListItem;
import com.persagy.apm.alarmservice.group.management.model.GroupManagement;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmitem.QueryAlarmItemDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.QueryAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmItem;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmRecordItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmItemServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmRecordServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 报警记录包装类，提供对报警相关字段的转换方法
 *
 * @author lixing
 * @version V1.0 2021/9/20 4:29 下午
 **/
@Service
public class AlarmRecordWrapper {
    @Autowired
    AlarmItemServiceImpl alarmItemService;
    @Autowired
    GroupManagementService groupManagementService;
    @Autowired
    AlarmRecordServiceImpl alarmRecordService;

    /**
     * 获取报警类型编码和名称的映射
     *
     * @param itemCodes 报警类型编码列表
     * @return 报警类型编码 -> simpleObjVO(id, name)
     * @throws
     * @author lixing
     * @version V1.0 2021/9/20 4:31 下午
     */
    public Map<String, SimpleObjVO> getAlarmItemMap(List<String> itemCodes) throws Exception {
        Map<String, SimpleObjVO> alarmItemMap = new HashMap<>();
        QueryAlarmItemDTO queryAlarmItemDTO = new QueryAlarmItemDTO();
        queryAlarmItemDTO.setCode(itemCodes);
        DmpResult<List<AlarmItem>> alarmItemsResult = alarmItemService.query(queryAlarmItemDTO);
        List<AlarmItem> alarmItems = alarmItemsResult.getData();
        if (CollectionUtils.isEmpty(alarmItems)) {

        }
        alarmItemMap.putAll(alarmItems.stream().collect(Collectors.toMap(
                AlarmItem::getId,
                alarmItem -> new SimpleObjVO(alarmItem.getId(), alarmItem.getName()),
                (k1, k2) -> k1
        )));
        return alarmItemMap;
    }

    /**
     * 获取两个时间的时间间隔（小时）
     *
     * @param triggerTime 触发时间
     * @param endTime     结束时间
     * @return 两个时间的时间间隔（小时）
     * @author lixing
     * @version V1.0 2021/9/20 4:45 下午
     */
    public static String getAlarmDuration(Date triggerTime, Date endTime) {
        if (endTime == null) {
            endTime = new Date();
        }
        if (triggerTime == null) {
            return null;
        }

        long durationNs = endTime.getTime() - triggerTime.getTime();
        // ns/1000=s s/60=min min/60=hour
        long durationHour = durationNs / (1000 * 60 * 60);
        // 保留两位小数
        BigDecimal bd = new BigDecimal(durationHour);
        bd = bd.setScale(2, RoundingMode.HALF_UP);
        return bd.toString();
    }

    /**
     * 将报警数据中台的查询结果转换为报警服务中台的报警记录列表
     *
     * @param dmpResult 报警数据中台的查询结果
     * @return 报警服务中台的报警记录列表
     * @author lixing
     * @version V1.0 2021/9/20 6:16 下午
     */
    public List<AlarmRecordListItem> getAlarmRecordListItems(DmpResult<List<AlarmRecordItem>> dmpResult) throws Exception {
        // 获取查询结果中的设备
        List<String> objIds = dmpResult.getData().stream().map(AlarmRecordItem::getObjId).collect(Collectors.toList());
        // 获取这些设备所在的分组信息
        Map<String, SimpleObjVO> objGroupMap = groupManagementService.queryObjGroup(objIds);
        // 获取这些设备的名称
        Map<String, GroupManagement> objMap = groupManagementService.queryEquipsByIds(objIds);
        if (CollectionUtils.isEmpty(dmpResult.getData())) {
            return new ArrayList<>();
        }
        // 获取查询结果中的报警类型
        List<String> itemCodes = dmpResult.getData().stream().map(AlarmRecordItem::getItemCode).
                collect(Collectors.toList());
        Map<String, SimpleObjVO> alarmItemMap = getAlarmItemMap(itemCodes);

        return dmpResult.getData().stream().map(alarmRecordItem ->
                getAlarmRecordListItem(objGroupMap, objMap, alarmItemMap, alarmRecordItem)).
                collect(Collectors.toList());
    }

    /**
     * 将报警数据中台返回的一条记录转换为报警服务中台报警记录列表中的一条数据
     *
     * @param objGroupMap     报警对象id和分组的映射关系
     * @param objMap          报警对象id和报警对象的映射关系
     * @param alarmItemMap    报警类型编码和报警类型的映射关系
     * @param alarmRecordItem 报警数据中台返回的一条报警记录
     * @return 报警服务中台报警记录列表中的一条数据
     * @author lixing
     * @version V1.0 2021/9/20 6:21 下午
     */
    private AlarmRecordListItem getAlarmRecordListItem(
            Map<String, SimpleObjVO> objGroupMap,
            Map<String, GroupManagement> objMap,
            Map<String, SimpleObjVO> alarmItemMap,
            AlarmRecordItem alarmRecordItem) {
        AlarmRecordListItem alarmRecordListItem = new AlarmRecordListItem();
        alarmRecordListItem.setId(alarmRecordItem.getId());
        alarmRecordListItem.setGroup(objGroupMap.get(alarmRecordItem.getObjId()));
        GroupManagement objInfo = objMap.get(alarmRecordItem.getObjId());
        alarmRecordListItem.setObj(new SimpleObjVO(alarmRecordItem.getObjId(),
                objInfo == null ? null : objInfo.getLocalName()));
        alarmRecordListItem.setAlarmItem(alarmItemMap.get(alarmRecordItem.getItemCode()));
        alarmRecordListItem.setTriggerTime(alarmRecordItem.getTriggerTime());
        // 计算报警的持续时长
        alarmRecordListItem.setDuration(
                AlarmRecordWrapper.getAlarmDuration(
                        alarmRecordItem.getTriggerTime(),
                        alarmRecordItem.getEndTime()
                )
        );
        EnumAlarmLevel alarmLevel = EnumAlarmLevel.getByType(alarmRecordItem.getLevel());
        if (alarmLevel != null) {
            alarmRecordListItem.setAlarmLevel(new SimpleObjVO(
                    alarmLevel.getType(), alarmLevel.getDesc()
            ));
        }

        EnumAlarmState alarmState = EnumAlarmState.getByType(
                String.valueOf(alarmRecordItem.getState()));
        if (alarmState != null) {
            alarmRecordListItem.setAlarmState(new SimpleObjVO(
                    alarmState.getType(), alarmState.getDesc()
            ));
        }

        EnumAlarmTreatState alarmTreatState = EnumAlarmTreatState.getByType(
                String.valueOf(alarmRecordItem.getTreatState()));
        if (alarmTreatState != null) {
            alarmRecordListItem.setAlarmTreatState(new SimpleObjVO(
                    alarmTreatState.getType(), alarmTreatState.getDesc()
            ));
        }

        EnumAlarmTreatMode alarmTreatMode = EnumAlarmTreatMode.getByType(
                String.valueOf(alarmRecordItem.getTreatMode()));
        if (alarmTreatMode != null) {
            alarmRecordListItem.setAlarmTreatMode(new SimpleObjVO(
                    alarmTreatMode.getType(), alarmTreatMode.getDesc()
            ));
        }

        return alarmRecordListItem;
    }

    /**
     * 查看报警记录详情
     *
     * @param recordId 报警记录id
     * @return 报警记录详情
     * @author lixing
     * @version V1.0 2021/9/21 3:54 下午
     */
    public AlarmRecordItem getAlarmRecord(String recordId) throws Exception {
        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO();
        queryAlarmRecordDTO.setId(Lists.newArrayList(recordId));
        DmpResult<List<AlarmRecordItem>> result = alarmRecordService.query(queryAlarmRecordDTO);
        if (result == null) {
            return null;
        }
        List<AlarmRecordItem> data = result.getData();
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        return data.get(0);
    }

    /**
     * 将报警数据中台的报警记录封装为业务中台的报警记录对象
     *
     * @param alarmRecordItem 数据中台的报警记录
     * @return 业务中台的报警记录对象
     * @author lixing
     * @version V1.0 2021/9/21 5:44 下午
     */
    public AlarmRecordItemVO convert2AlarmRecordItemVO(AlarmRecordItem alarmRecordItem) throws Exception {
        if (alarmRecordItem == null) {
            return null;
        }
        AlarmRecordItemVO alarmRecordItemVO = new AlarmRecordItemVO();
        /* 报警基本信息 */
        copyBasicInfo(alarmRecordItem, alarmRecordItemVO);
        /* 报警条件 */
        JSONObject condition = alarmRecordItem.getCondition();
        AlarmConditionStrItem alarmConditionStrItem = new AlarmConditionStrItem();
        // 报警规则
        String rules = condition.getString("rules");
        List<String> rulesList = JSONArray.parseArray(rules, String.class);
        alarmConditionStrItem.setRules(rulesList);
        // 报警触发时间转换为分钟
        Integer triggerUphold = condition.getInteger("triggerUphold");
        if (triggerUphold != null) {
            triggerUphold = triggerUphold / 60;
        }
        String lastTimeInfo = "持续%s分钟满足报警触发规则，则产生报警";
        alarmConditionStrItem.setLastTimeInfo(
                String.format(lastTimeInfo, triggerUphold == null ? "--" : triggerUphold));
        EnumAlarmLevel alarmLevel = EnumAlarmLevel.getByType(alarmRecordItem.getLevel());
        if (alarmLevel != null) {
            alarmConditionStrItem.setAlarmLevel(new SimpleObjVO(
                    alarmLevel.getType(), alarmLevel.getDesc()
            ));
        }
        alarmRecordItemVO.setAlarmCondition(alarmConditionStrItem);
        return alarmRecordItemVO;
    }

    /**
     * 将报警数据中台报警对象的基本信息复制到报警业务中台报警对象
     *
     * @param alarmRecordItem   报警数据中台报警对象
     * @param alarmRecordItemVO 报警业务中台报警对象
     * @author lixing
     * @version V1.0 2021/9/22 3:41 下午
     */
    public void copyBasicInfo(AlarmRecordItem alarmRecordItem, AlarmRecordItemVO alarmRecordItemVO) throws Exception {
        alarmRecordItemVO.setId(alarmRecordItem.getId());
        Map<String, SimpleObjVO> objGroupMap = groupManagementService.queryObjGroup(
                Lists.newArrayList(alarmRecordItem.getObjId()));
        alarmRecordItemVO.setGroup(objGroupMap.get(alarmRecordItem.getObjId()));
        // 获取这些设备的名称
        Map<String, GroupManagement> objMap = groupManagementService.queryEquipsByIds(
                Lists.newArrayList(alarmRecordItem.getObjId())
        );
        GroupManagement objInfo = objMap.get(alarmRecordItem.getObjId());
        alarmRecordItemVO.setObj(new SimpleObjVO(alarmRecordItem.getObjId(),
                objInfo == null ? null : objInfo.getLocalName()));
        // 报警类型
        Map<String, SimpleObjVO> alarmItemMap = getAlarmItemMap(Lists.newArrayList(alarmRecordItem.getItemCode()));
        SimpleObjVO simpleObjVO = alarmItemMap.get(alarmRecordItem.getItemCode());
        alarmRecordItemVO.setAlarmItem(simpleObjVO);

        alarmRecordItemVO.setTriggerTime(alarmRecordItem.getTriggerTime());
        alarmRecordItemVO.setEndTime(alarmRecordItem.getEndTime());
        alarmRecordItemVO.setDuration(getAlarmDuration(alarmRecordItem.getTriggerTime(), alarmRecordItem.getEndTime()));

        EnumAlarmLevel alarmLevel = EnumAlarmLevel.getByType(alarmRecordItem.getLevel());
        if (alarmLevel != null) {
            alarmRecordItemVO.setAlarmLevel(new SimpleObjVO(
                    alarmLevel.getType(), alarmLevel.getDesc()
            ));
        }

        EnumAlarmState alarmState = EnumAlarmState.getByType(
                String.valueOf(alarmRecordItem.getState()));
        if (alarmState != null) {
            alarmRecordItemVO.setAlarmState(new SimpleObjVO(
                    alarmState.getType(), alarmState.getDesc()
            ));
        }

        EnumAlarmTreatState alarmTreatState = EnumAlarmTreatState.getByType(
                String.valueOf(alarmRecordItem.getTreatState()));
        if (alarmTreatState != null) {
            alarmRecordItemVO.setAlarmTreatState(new SimpleObjVO(
                    alarmTreatState.getType(), alarmTreatState.getDesc()
            ));
        }

        EnumAlarmTreatMode alarmTreatMode = EnumAlarmTreatMode.getByType(
                String.valueOf(alarmRecordItem.getTreatMode()));
        if (alarmTreatMode != null) {
            alarmRecordItemVO.setAlarmTreatMode(new SimpleObjVO(
                    alarmTreatMode.getType(), alarmTreatMode.getDesc()
            ));
        }
        /* 近30天报警触发次数 */
        Integer count = getAlarmCountInMonth(alarmRecordItem.getTriggerTime(), alarmRecordItem.getItemCode());
        alarmRecordItemVO.setAlarmCountInMonth(count);
    }

    /**
     * 指定报警类型在指定日期前一个月内发生报警的次数
     *
     * @param date     指定日期
     * @param itemCode 指定报警类型
     * @return 发生报警的次数
     * @author lixing
     * @version V1.0 2021/9/22 3:43 下午
     */
    public Integer getAlarmCountInMonth(Date date, String itemCode) throws Exception {
        if (date == null || StringUtils.isBlank(itemCode)) {
            return 0;
        }
        QueryAlarmRecordDTO queryAlarmRecordDTO = new QueryAlarmRecordDTO();
        queryAlarmRecordDTO.setTriggerDurationStart(
                DateUtils.getMonthOff(date, -1));
        queryAlarmRecordDTO.setTriggerDurationEnd(date);
        queryAlarmRecordDTO.setItemCode(Lists.newArrayList(itemCode));
        return alarmRecordService.queryCount(queryAlarmRecordDTO);
    }

}
